<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                exclude-result-prefixes="xalan lav">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="lav:*"/>

    <!-- AppendAggregateProcessor -->
    <xsl:template match="lav:processor[contains(@type,'AppendAggregateProcessor')]/@match">
        <xsl:attribute name="match">
            <xsl:value-of select="."/> | <xsl:value-of select="."/>/<xsl:value-of
                select="parent::lav:processor/lav:parameter[@name='node_values']/@eval"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="lav:processor[contains(@type,'AppendAggregateProcessor')]/lav:parameter[@name='node_values']"/>

    <!-- MoveProcessor / CopyProcessor -->
    <xsl:template match="lav:processor[contains(@type,'MoveProcessor') or contains(@type,'CopyProcessor')]/@match">
        <xsl:variable name="match">
            <xsl:call-template name="REMOVE_PREDICATE"><xsl:with-param name="path" select="."/></xsl:call-template>
        </xsl:variable>
        <xsl:attribute name="match">
            <xsl:value-of select="."/>
            <xsl:if test="not(contains($match,'/@') or contains($match,'/attribute::')
                       or contains($match,'/text()') or contains($match,'/comment()'))">[* or not(*)]</xsl:if>
        </xsl:attribute>
    </xsl:template>

    <!-- SelectGroupByProcessor -->
    <xsl:template match="lav:processor[contains(@type,'SelectGroupByProcessor')]/@match">
        <xsl:attribute name="match">
            <xsl:text>/* | </xsl:text>
            <xsl:value-of select="."/>/<xsl:value-of select="parent::lav:processor/lav:parameter[@name='group_by']/@eval"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="lav:processor[contains(@type,'SelectGroupByProcessor')]/lav:parameter[@name='group_by']/@eval">
        <xsl:attribute name="eval">
            <xsl:text>.</xsl:text>
            <xsl:call-template name="REVERT_PATH">
                <xsl:with-param name="path">
                    <xsl:call-template name="REMOVE_PREDICATE">
                        <xsl:with-param name="path" select="."/>
                    </xsl:call-template>
                </xsl:with-param>
            </xsl:call-template>
            <xsl:text>/parent::*</xsl:text>
        </xsl:attribute>
    </xsl:template>

    <xsl:template name="REMOVE_PREDICATE">
        <xsl:param name="path"/>
        <xsl:choose>
            <xsl:when test="contains($path, '[')">
                <xsl:call-template name="REMOVE_PREDICATE">
                    <xsl:with-param name="path" select="concat(substring-before($path,'['), substring-after($path,']'))"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$path"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="REVERT_PATH">
        <xsl:param name="path"/>
        <xsl:if test="contains($path, '/')">
            <xsl:call-template name="REVERT_PATH">
                <xsl:with-param name="path" select="substring-after($path, '/')"/>
            </xsl:call-template>/parent::<xsl:value-of select="substring-before($path, '/')"/>
        </xsl:if>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="@*">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*">
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>