<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                exclude-result-prefixes="wsdl xsd">
    <!-- This template creates a view for invoking the specified WSDL service -->
    <xsl:template match="lav:view-template[contains(@stylesheet,'WSDLTemplate.xsl')]">
        <!-- template parameters -->
        <xsl:variable name="wsdl_url">
            <xsl:call-template name="PARAMETER">
                <xsl:with-param name="name">wsdl</xsl:with-param>
                <xsl:with-param name="default"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="wsdl" select="document($wsdl_url)
            | document(document($wsdl_url)/descendant::wsdl:import/@location)
            | document(document($wsdl_url)/descendant::xsd:import/@schemaLocation)
            | document(document(document($wsdl_url)/descendant::wsdl:import/@location)/descendant::xsd:import/@schemaLocation)"/>
        <xsl:variable name="port">
            <xsl:call-template name="PARAMETER">
                <xsl:with-param name="name">port</xsl:with-param>
                <xsl:with-param name="default" select="$wsdl/wsdl:definitions/wsdl:service[1]/wsdl:port[1]/@name"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="operation">
            <xsl:call-template name="PARAMETER">
                <xsl:with-param name="name">operation</xsl:with-param>
                <xsl:with-param name="default">
                    <xsl:for-each select="$wsdl/wsdl:definitions/wsdl:service/wsdl:port[@name=$port]">
                        <xsl:for-each select="$wsdl/wsdl:definitions/wsdl:binding[@name=substring-after(current()/@binding,':')]">
                            <xsl:value-of select="$wsdl/wsdl:definitions/wsdl:portType[@name=substring-after(current()/@type,':')]/wsdl:operation[1]/@name"/>
                        </xsl:for-each>
                    </xsl:for-each>
                </xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="SOAPHeader">
            <xsl:choose>
                <xsl:when test="lav:parameter[@name='SOAPHeader']/@eval"><xsl:value-of select="lav:parameter[@name='SOAPHeader']/@eval"/></xsl:when>
                <xsl:otherwise>'<xsl:value-of select="lav:parameter[@name='SOAPHeader']/text()"/>'</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <!-- generated view -->
        <view>
            <xsl:copy-of select="@*[not(local-name()='stylesheet')] | lav:*[not(self::lav:parameter)][not(self::lav:fallback)]"/>
            <!-- arguments -->
            <xsl:for-each select="$wsdl/wsdl:definitions/wsdl:service/wsdl:port[@name=$port]">
                <xsl:for-each select="$wsdl/wsdl:definitions/wsdl:binding[@name=substring-after(current()/@binding,':')]">
                    <xsl:for-each select="$wsdl/wsdl:definitions/wsdl:portType[@name=substring-after(current()/@type,':')]/wsdl:operation[@name=$operation]/wsdl:input">
                        <xsl:for-each select="$wsdl/wsdl:definitions/wsdl:message[@name=substring-after(current()/@message,':')]/wsdl:part">
                            <xsl:for-each select="$wsdl/descendant::xsd:schema/xsd:element[@name=substring-after(current()/@element,':')]">
                                <xsl:apply-templates select="self::xsd:element" mode="ARGUMENTS">
                                    <xsl:with-param name="wsdl" select="$wsdl"/>
                                </xsl:apply-templates>
                            </xsl:for-each>
                        </xsl:for-each>
                    </xsl:for-each>
                </xsl:for-each>
            </xsl:for-each>

            <!-- plugins -->
            <connector type="HTTPConnector">
                <parameter name="url">
                    <xsl:value-of select="$wsdl/wsdl:definitions/wsdl:service/wsdl:port[@name=$port]/*/@location"/>
                </parameter>
                <parameter name="header">
                    <entry key="Content-Type">text/xml; charset=ISO-8859-1</entry>
                    <entry key="SOAPAction">
                        <xsl:variable name="binding" select="substring-after($wsdl/wsdl:definitions/wsdl:service/wsdl:port[@name=$port]/@binding,':')"/>
                        <xsl:value-of select="$wsdl/wsdl:definitions/wsdl:binding[@name=$binding]/wsdl:operation[@name=$operation]/*/@soapAction"/>
                    </entry>
                </parameter>
                <parameter name="post">
                    <xsl:attribute name="eval">concat(<xsl:text/>
'&lt;SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"&gt;',
'&lt;SOAP-ENV:Header&gt;',
<xsl:value-of select="$SOAPHeader"/>,
'&lt;/SOAP-ENV:Header&gt;',
'&lt;SOAP-ENV:Body&gt;',
<xsl:for-each select="$wsdl/wsdl:definitions/wsdl:service/wsdl:port[@name=$port]">
<xsl:for-each select="$wsdl/wsdl:definitions/wsdl:binding[@name=substring-after(current()/@binding,':')]">
<xsl:for-each select="$wsdl/wsdl:definitions/wsdl:portType[@name=substring-after(current()/@type,':')]/wsdl:operation[@name=$operation]/wsdl:input">
    <xsl:for-each select="$wsdl/wsdl:definitions/wsdl:message[@name=substring-after(current()/@message,':')]/wsdl:part">
        <xsl:for-each select="$wsdl/descendant::xsd:schema/xsd:element[@name=substring-after(current()/@element,':')]">
            <xsl:apply-templates select="self::xsd:element" mode="SOAP">
                <xsl:with-param name="wsdl" select="$wsdl"/>
                <xsl:with-param name="namespace" select="parent::xsd:schema/@targetNamespace"/>
            </xsl:apply-templates>
        </xsl:for-each>
    </xsl:for-each>
</xsl:for-each>
</xsl:for-each>
</xsl:for-each>
'&lt;/SOAP-ENV:Body&gt;',
'&lt;/SOAP-ENV:Envelope&gt;'
)<xsl:text/>
                    </xsl:attribute>
                </parameter>
                <xsl:copy-of select="lav:fallback"/>
            </connector>
            <processors>
                <processor type="RemoveProcessor" match="/soapenv:Envelope">
                    <namespaces>
                        <entry key="soapenv">http://schemas.xmlsoap.org/soap/envelope/</entry>
                    </namespaces>
                    <parameter name="depth">2</parameter>
                </processor>
            </processors>
        </view>
    </xsl:template>

    <xsl:template name="PARAMETER">
        <xsl:param name="name"/>
        <xsl:param name="default"/>
        <xsl:choose>
            <xsl:when test="lav:parameter[@name=$name]/text()">
                <xsl:value-of select="lav:parameter[@name=$name]/text()"/>
            </xsl:when>
            <xsl:when test="$default">
                <xsl:value-of select="$default"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message terminate="yes">View template '<xsl:value-of
                        select="@stylesheet"/>' is missing required parameter: <xsl:value-of select="$name"/>
                </xsl:message>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="xsd:element" mode="ARGUMENTS">
        <xsl:param name="wsdl"/>
        <xsl:param name="position"/>
        <xsl:choose>
            <xsl:when test="contains(@type,':') and $wsdl/descendant::xsd:schema/xsd:complexType[@name=substring-after(current()/@type,':')]">
                <xsl:apply-templates select="$wsdl/descendant::xsd:schema/xsd:complexType[@name=substring-after(current()/@type,':')]" mode="ARGUMENTS">
                    <xsl:with-param name="wsdl" select="$wsdl"/>
                    <xsl:with-param name="position" select="concat($position, position())"/>
                </xsl:apply-templates>
            </xsl:when>
            <xsl:when test="@type">
                <argument name="{@name}_{$position}"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="*" mode="ARGUMENTS">
                    <xsl:with-param name="wsdl" select="$wsdl"/>
                    <xsl:with-param name="position" select="concat($position, position())"/>
                </xsl:apply-templates>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="*" mode="ARGUMENTS">
        <xsl:param name="wsdl"/>
        <xsl:param name="position"/>
        <xsl:apply-templates select="*" mode="ARGUMENTS">
            <xsl:with-param name="wsdl" select="$wsdl"/>
            <xsl:with-param name="position" select="$position"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="xsd:element" mode="SOAP">
        <xsl:param name="wsdl"/>
        <xsl:param name="position"/>
        <xsl:param name="namespace"/>
        <xsl:choose>
            <xsl:when test="contains(@type,':') and $wsdl/descendant::xsd:schema/xsd:complexType[@name=substring-after(current()/@type,':')]">
'&lt;<xsl:value-of select="@name"/><xsl:if test="$namespace"> xmlns="<xsl:value-of select="$namespace"/>"</xsl:if>&gt;',
                <xsl:apply-templates select="$wsdl/descendant::xsd:schema/xsd:complexType[@name=substring-after(current()/@type,':')]" mode="SOAP">
                    <xsl:with-param name="wsdl" select="$wsdl"/>
                    <xsl:with-param name="position" select="concat($position, position())"/>
                </xsl:apply-templates>
'&lt;/<xsl:value-of select="@name"/>&gt;',
            </xsl:when>
            <xsl:when test="@type">
                <xsl:variable name="arg">arguments()[@key='<xsl:value-of select="@name"/>_<xsl:value-of select="$position"/>']</xsl:variable>
                <xsl:text/>choose(<xsl:value-of select="$arg"/>, concat('&lt;<xsl:value-of select="@name"/>&gt;',<xsl:value-of select="$arg"/>,'&lt;/<xsl:value-of select="@name"/>&gt;')),<xsl:text/>
            </xsl:when>
            <xsl:otherwise>
'&lt;<xsl:value-of select="@name"/><xsl:if test="$namespace"> xmlns="<xsl:value-of select="$namespace"/>"</xsl:if>&gt;',
                <xsl:apply-templates select="*" mode="SOAP">
                    <xsl:with-param name="wsdl" select="$wsdl"/>
                    <xsl:with-param name="position" select="concat($position, position())"/>
                </xsl:apply-templates>
'&lt;/<xsl:value-of select="@name"/>&gt;',
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="*" mode="SOAP">
        <xsl:param name="wsdl"/>
        <xsl:param name="position"/>
        <xsl:apply-templates select="*" mode="SOAP">
            <xsl:with-param name="wsdl" select="$wsdl"/>
            <xsl:with-param name="position" select="$position"/>
        </xsl:apply-templates>
    </xsl:template>
</xsl:stylesheet>