<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd">
    <!-- This template enables querying a database with XPath expressions -->
    <xsl:template match="lav:view-template[contains(@stylesheet,'XPath2SQLTemplate.xsl')]">
        <view><xsl:copy-of select="@*[not(local-name()='stylesheet')] | lav:*[not(self::lav:parameter)]"/>
            <argument name="disable-default-XPath" path-format="value">true</argument>
            <connector type="SQLConnector">
                <xsl:copy-of select="lav:parameter[not(@name='query')]"/>
                <parameter name="query" eval="view_path('{@name}_xpath2sql', path())"/>
            </connector>
        </view>
        <view name="{@name}_xpath2sql">
            <xsl:copy-of select="lav:*[not(self::lav:parameter)]"/>
            <argument name="disable-default-XPath" path-format="value">true</argument>
            <connector type="XPath2SQLConnector">
                <parameter name="foreign-keys" eval="view('{@name}_foreign-keys')"/>
                <parameter name="xpath" eval="path()"/>
            </connector>
        </view>
        <view name="{@name}_foreign-keys">
            <namespace prefix="e" uri="http://software.in2p3.fr/lavoisier/entries.xsd"/>
            <xsl:copy-of select="lav:*[not(self::lav:parameter)]"/>
            <connector type="SQLConnector">
                <xsl:copy-of select="lav:parameter[not(@name='query')]"/>
                <parameter name="query">
                    SELECT CONCAT(CONCAT(fk.TABLE_NAME, '.'), fk.COLUMN_NAME) AS "foreign-key",
                    CONCAT(CONCAT(ref.TABLE_NAME, '.'), ref.COLUMN_NAME) AS "references"
                    FROM INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS link
                    LEFT OUTER JOIN INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE fk
                    ON link.CONSTRAINT_NAME = fk.CONSTRAINT_NAME
                    LEFT OUTER JOIN INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE ref
                    ON link.UNIQUE_CONSTRAINT_NAME = ref.CONSTRAINT_NAME;
                </parameter>
            </connector>
            <processors>
                <processor  type="MultiMergeProcessor" match="/result/row">
                    <parameter name="count">2</parameter>
                </processor>
                <processor type="RenameProcessor" match="/result/row/@row">
                    <parameter name="node_name">key</parameter>
                </processor>
                <processor type="RenameProcessor" match="/result/row">
                    <parameter name="node_name">e:entry</parameter>
                </processor>
                <processor type="RenameProcessor" match="/result">
                    <parameter name="node_name">e:entries</parameter>
                </processor>
            </processors>
            <cache type="FileCache">
                <trigger type="ViewCreatedTrigger"/>
                <trigger type="ViewNotifiedTrigger" ignore-during="PT0S"/>
            </cache>
        </view>
    </xsl:template>
</xsl:stylesheet>