<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                exclude-result-prefixes="cfg">
    <xsl:output method="xml" omit-xml-declaration="yes"/>
    <xsl:variable name="VERSION">13</xsl:variable>

    <!-- https://forge.in2p3.fr/issues/6265 -->
    <xsl:template match="@post-processors[.='disable-default-processors']">
        <xsl:attribute name="post-processors">disable</xsl:attribute>
    </xsl:template>

    <!-- https://forge.in2p3.fr/issues/6363 -->
    <xsl:template match="cfg:insert[@as='attribute'] | cfg:to-ancestor[@as='attribute'] | cfg:to-following-sibling[@as='attribute']
                       | cfg:processor[contains(@type,'InsertProcessor') or contains(@type,'MoveProcessor') or contains(@type,'CopyProcessor')]
                         /cfg:parameter[@name='destination_as' and text()='attribute']">
        <xsl:message terminate="yes">
            Parameter 'as' does not support value 'attribute' anymore.
            Please explicitly convert nodes to attribute before inserting/moving/copying them.
        </xsl:message>
    </xsl:template>
    <xsl:template match="@as[.='attribute'] | cfg:parameter[@name='destination_as' and text()='attribute']"/>

    <!-- https://forge.in2p3.fr/issues/6266 -->
    <xsl:template match="@*[contains(.,'view(concat(')] | text()[contains(.,'view(concat(')]" priority="3">
        <xsl:message terminate="yes">Please use view_path() instead of view(concat())</xsl:message>
    </xsl:template>
    <xsl:template match="@*[contains(.,'view_post(concat(')] | text()[contains(.,'view_post(concat(')]" priority="3">
        <xsl:message terminate="yes">Please use view_path_post() instead of view_post(concat())</xsl:message>
    </xsl:template>
    <xsl:template match="@*[contains(.,'view(')]">
        <xsl:attribute name="{local-name()}">
            <xsl:call-template name="SPLIT_VIEW_AND_PATH">
                <xsl:with-param name="function">view(</xsl:with-param>
            </xsl:call-template>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="@*[contains(.,'view_post(')]">
        <xsl:attribute name="{local-name()}">
            <xsl:call-template name="SPLIT_VIEW_AND_PATH">
                <xsl:with-param name="function">view_post(</xsl:with-param>
            </xsl:call-template>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="text()[contains(.,'view(')]">
        <xsl:call-template name="SPLIT_VIEW_AND_PATH">
            <xsl:with-param name="function">view(</xsl:with-param>
        </xsl:call-template>
    </xsl:template>
    <xsl:template match="text()[contains(.,'view_post(')]">
        <xsl:call-template name="SPLIT_VIEW_AND_PATH">
            <xsl:with-param name="function">view_post(</xsl:with-param>
        </xsl:call-template>
    </xsl:template>
    <xsl:template name="SPLIT_VIEW_AND_PATH">
        <xsl:param name="function"/>
        <xsl:variable name="new_function">
            <xsl:choose>
                <xsl:when test="$function='view('">view_path</xsl:when>
                <xsl:when test="$function='view_post('">view_path_post</xsl:when>
                <xsl:otherwise><xsl:message terminate="yes">INTERNAL ERROR</xsl:message></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="args" select="substring-before(substring-after(.,$function),')')"/>
        <xsl:choose>
            <xsl:when test="contains($args,'/')">
                <xsl:value-of select="substring-before(.,$function)"/>
                <xsl:value-of select="$new_function"/>
                <xsl:text>(</xsl:text>
                <xsl:value-of select="substring-before($args,'/')"/>', '/<xsl:value-of select="substring-after($args,'/')"/>
                <xsl:text>)</xsl:text>
                <xsl:value-of select="substring-after(substring-after(.,$function),')')"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="."/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- https://forge.in2p3.fr/issues/6705 -->
    <xsl:template match="cfg:processor/@type[.='RemoveNamespaceProcessor']">
        <xsl:attribute name="type">ChangeNamespaceProcessor</xsl:attribute>
    </xsl:template>

    <!-- https://forge.in2p3.fr/issues/6691 -->
    <xsl:template match="cfg:processor/@type[.='NodeExistValidator']">
        <xsl:attribute name="type">XPathValidatorProcessor</xsl:attribute>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='NodeExistValidator']/cfg:parameter/@name[.='fail-when']">
        <xsl:attribute name="name">exist</xsl:attribute>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='NodeExistValidator']/cfg:parameter[@name='fail-when']/@eval">
        <xsl:attribute name="eval">not(<xsl:value-of select="."/>)</xsl:attribute>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='NodeExistValidator']/cfg:parameter[@name='fail-when']/text()[.!='']">
        <xsl:value-of select="string(not(.))"/>
    </xsl:template>

    <!-- https://forge.in2p3.fr/issues/6704 -->
    <xsl:template match="@*[contains(.,'user(')]">
        <xsl:value-of select="substring-before(.,'user(')"/>user()<xsl:value-of select="substring-after(substring-after(.,'user('),')')"/>
    </xsl:template>
    <xsl:template match="text()[contains(.,'user(')]">
        <xsl:value-of select="substring-before(.,'user(')"/>user()<xsl:value-of select="substring-after(substring-after(.,'user('),')')"/>
    </xsl:template>

    <!-- https://forge.in2p3.fr/issues/6759 -->
    <xsl:template match="cfg:view[@authorized]">
        <xsl:message terminate="yes">Please move attribute @authorized to an authenticators section</xsl:message>
    </xsl:template>

    <!-- https://forge.in2p3.fr/issues/6326 -->
    <xsl:template match="cfg:processor[cfg:namespace]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <namespaces>
                <xsl:apply-templates select="cfg:namespace"/>
            </namespaces>
            <xsl:apply-templates select="cfg:*[not(self::cfg:namespace)] | text() | comment()"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="cfg:processor/cfg:namespace">
        <entry key="{@prefix}"><xsl:value-of select="@uri"/></entry>
    </xsl:template>

    <!-- https://forge.in2p3.fr/issues/6930 -->
    <xsl:template match="cfg:trigger[starts-with(@type,'DeltaTimeAnd') or contains(@type,'AndDeltaTime')]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:attribute name="ignore-during">
                <xsl:text>P</xsl:text>
                <xsl:for-each select="cfg:parameter[@name='years']"><xsl:value-of select="text()"/>Y</xsl:for-each>
                <xsl:for-each select="cfg:parameter[@name='months']"><xsl:value-of select="text()"/>M</xsl:for-each>
                <xsl:for-each select="cfg:parameter[@name='days']"><xsl:value-of select="text()"/>D</xsl:for-each>
                <xsl:text>T</xsl:text>
                <xsl:for-each select="cfg:parameter[@name='hours']"><xsl:value-of select="text()"/>H</xsl:for-each>
                <xsl:for-each select="cfg:parameter[@name='minutes']"><xsl:value-of select="text()"/>M</xsl:for-each>
                <xsl:for-each select="cfg:parameter[@name='seconds']"><xsl:value-of select="text()"/>S</xsl:for-each>
            </xsl:attribute>
            <xsl:apply-templates select="cfg:parameter[not(contains('|years|months|days|hours|minutes|seconds|',concat('|',@name,'|')))]"/>
        </xsl:copy>
    </xsl:template>

    <!-- common -->
    <xsl:template match="/cfg:config/@version">
        <xsl:attribute name="version">
            <xsl:value-of select="$VERSION"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="@* | comment()" priority="-1">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*" priority="-1">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>