<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns="http://software.in2p3.fr/lavoisier/tables.xsd"
                exclude-result-prefixes="xalan">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/>

    <xsl:template match="/parameters">
        <options>
            <xsl:apply-templates select="type|stacking|axis"/>
            <xsl:apply-templates select="xAxis|yAxis|yAxis2|yAxis3"/>
            <xsl:apply-templates select="unit|stacked|types|drilldown"/>
            <xsl:apply-templates select="options/options"/>
        </options>
    </xsl:template>

    <xsl:template match="type">
        <xsl:attribute name="type"><xsl:value-of select="text()"/></xsl:attribute>
    </xsl:template>
    <xsl:template match="stacking">
        <xsl:attribute name="stacking"><xsl:value-of select="text()"/></xsl:attribute>
    </xsl:template>
    <xsl:template match="axis">
        <xsl:attribute name="axis"><xsl:value-of select="text()"/></xsl:attribute>
    </xsl:template>

    <xsl:template match="xAxis">
        <xsl:copy>
            <xsl:for-each select="entry">
                <column><xsl:value-of select="text()"/></column>
            </xsl:for-each>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="yAxis|yAxis2|yAxis3">
        <xsl:copy>
            <xsl:attribute name="id">
                <xsl:apply-templates select="." mode="POSITION"/>
            </xsl:attribute>
            <xsl:for-each select="/parameters/unit/entry[@key=local-name(current())]">
                <xsl:attribute name="unit"><xsl:value-of select="text()"/></xsl:attribute>
            </xsl:for-each>
            <xsl:for-each select="/parameters/stacked/entry[@key=local-name(current())]">
                <xsl:attribute name="stacked"><xsl:value-of select="text()"/></xsl:attribute>
            </xsl:for-each>
            <xsl:for-each select="entry">
                <column>
                    <xsl:for-each select="/parameters/types/entry[@key=current()/text()]">
                        <xsl:attribute name="type"><xsl:value-of select="text()"/></xsl:attribute>
                    </xsl:for-each>
                    <xsl:for-each select="/parameters/drilldown/entry[@key=current()/text()]">
                        <xsl:attribute name="drilldown"><xsl:value-of select="text()"/></xsl:attribute>
                    </xsl:for-each>
                    <xsl:value-of select="text()"/>
                </column>
            </xsl:for-each>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="unit|stacked|types|drilldown">
        <xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="options">
        <sections>
            <xsl:apply-templates mode="OPTIONS"/>
        </sections>
    </xsl:template>
    <xsl:template match="yAxis|yAxis2|yAxis3" mode="OPTIONS">
        <section id="yAxis">
            <xsl:attribute name="position">
                <xsl:apply-templates select="." mode="POSITION"/>
            </xsl:attribute>
            <xsl:apply-templates mode="OPTIONS"/>
        </section>
    </xsl:template>
    <xsl:template match="*[not(text())]" mode="OPTIONS" priority="-1">
        <section id="{local-name()}">
            <xsl:apply-templates mode="OPTIONS"/>
        </section>
    </xsl:template>
    <xsl:template match="*[text()]" mode="OPTIONS" priority="-1">
        <option name="{local-name()}">
            <xsl:value-of select="text()"/>
        </option>
    </xsl:template>

    <xsl:template match="yAxis|yAxis2|yAxis3" mode="POSITION">
        <xsl:choose>
            <xsl:when test="self::yAxis">1</xsl:when>
            <xsl:otherwise><xsl:value-of select="substring-after(local-name(),'yAxis')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>