<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:t="http://software.in2p3.fr/lavoisier/tables.xsd">
    <xsl:output method="html"  indent="yes" />

    <xsl:variable name="options" select="document('options')/t:options"/>

    <xsl:variable name="type" select="$options/@type"/>
    <xsl:variable name="stacking" select="$options/@stacking"/>
    <xsl:variable name="axis" select="$options/@axis"/>
    <xsl:variable name="unit" select="$options/t:unit/t:entry"/>
    <xsl:variable name="stacked" select="$options/t:stacked/t:entry"/>
    <xsl:variable name="types" select="$options/t:types/t:entry"/>
    <xsl:variable name="drilldown" select="$options/t:drilldown/t:entry"/>
    <xsl:variable name="sections" select="$options/t:sections/t:section"/>

    <!-- prevents unwanted text() nodes to be printed -->
    <xsl:template match="/t:tables">
<html>
<head>
    <xsl:for-each select="/t:tables/@title">
        <title><xsl:value-of select="."/></title>
    </xsl:for-each>
    <script type="text/javascript" src="/resource/js/jquery.min.js"></script>
    <script type="text/javascript">
$(function () {
    $(document).ready(function() {
        var json;
        <xsl:for-each select="t:table">
            eval("json = "+document.getElementById('data_<xsl:value-of select="@name"/>').innerHTML);
            new Highcharts.Chart(json);
        </xsl:for-each>
    });
});
    </script>
</head>
<body>
    <script src="/resource/js/highcharts.js"></script>
    <script src="/resource/js/highcharts-more.js"></script>
    <script src="/resource/js/modules/exporting.js"></script>
    <xsl:for-each select="t:table">
        <div id="container_{@name}" style="min-width: 400px; height: 400px; margin: 0 auto"></div>
    </xsl:for-each>
    <xsl:for-each select="t:table">
        <div id="data_{@name}" style="display: none">
            <xsl:apply-templates select="."/>
        </div>
    </xsl:for-each>
</body>
</html>
    </xsl:template>

    <xsl:template match="t:table">
        <xsl:variable name="highcharts-stacking">
            <xsl:choose>
                <xsl:when test="$stacking='none'">null</xsl:when>
                <xsl:when test="$stacking='normal'">'normal'</xsl:when>
                <xsl:when test="$stacking='percent'">'percent'</xsl:when>
                <xsl:otherwise>
                    <xsl:message terminate="yes">Unsupported chart stacking: <xsl:value-of select="$stacking"/></xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="$axis='normal'"/>
            <xsl:when test="$axis='inverted'"/>
            <xsl:when test="$axis='polar'"/>
            <xsl:otherwise>
                <xsl:message terminate="yes">Unsupported chart axis: <xsl:value-of select="$axis"/></xsl:message>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:variable name="highcharts-units" select="ancestor-or-self::t:table/t:column_labels/t:column_label[not(preceding-sibling::t:column_label/@unit = @unit)]"/>
        <xsl:variable name="highcharts-colors" xmlns:str="http://exslt.org/strings"
                      select="str:split('4572A7,AA4643,89A54E,80699B,3D96AE,DB843D,92A8CD,A47D7C,B5CA92,4572A7', ',')"/>

        {
            chart: {
                renderTo: 'container_<xsl:value-of select="@name"/>',
                inverted: <xsl:value-of select="string($axis='inverted')"/>,
                polar: <xsl:value-of select="string($axis='polar')"/>
                <xsl:apply-templates select="$sections[@id='chart']/*"/>
            },
            plotOptions: {
        <xsl:value-of select="$type"/>: {
        <xsl:if test="$type='column' and $axis='polar'">
                    pointPadding: 0,
                    groupPadding: 0,
        </xsl:if>
                    stacking: <xsl:value-of select="$highcharts-stacking"/>
                }
            },

            title: {
                text: '<xsl:value-of select="/t:tables/@title"/> <xsl:value-of select="@name"/>'
            },
        <xsl:for-each select="/t:tables/t:fields">
            subtitle: {
                text: '<xsl:for-each select="t:field">
                <xsl:if test="position() > 1">, </xsl:if>
                <xsl:choose>
                    <xsl:when test="@link">
                        <a href="{@link}"><xsl:value-of select="text()"/></a><xsl:text/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="text()"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>'
            },
        </xsl:for-each>
            xAxis: {
                categories: [
        <xsl:for-each select="t:row">
            <xsl:if test="position()>1">, </xsl:if>
            <xsl:text>'</xsl:text>
            <xsl:variable name="row" select="."/>
            <xsl:for-each select="ancestor-or-self::t:table/t:column_labels/t:column_label[not(@unit)]">
                <xsl:if test="position()>1">_</xsl:if>
                <xsl:value-of select="$row/t:column[count(current()/preceding-sibling::t:column_label)+1]/text()"/>
            </xsl:for-each>
            <xsl:text>'</xsl:text>
        </xsl:for-each>
                ],
        <xsl:if test="$type!='column' and $axis='polar'">
                tickmarkPlacement: 'on',
        </xsl:if>
                title: {
        <xsl:if test="$axis!='polar'">
                    text: '<xsl:for-each select="ancestor-or-self::t:table/t:column_labels/t:column_label[not(@unit)]">
            <xsl:if test="position()>1">_</xsl:if>
            <xsl:value-of select="text()"/>
            </xsl:for-each>',
        </xsl:if>
                    align: 'high'
                }
        <xsl:apply-templates select="$sections[@id='xAxis']/*"/>
            },
            yAxis: [
        <xsl:for-each select="$highcharts-units">
            <xsl:variable name="unit" select="@unit"/>
            <xsl:variable name="color" select="$highcharts-colors
[position() = count(ancestor-or-self::t:table/t:column_labels/t:column_label[@unit=$unit][1]/preceding-sibling::t:column_label[@unit])+1]"/>
            <xsl:if test="position()>1">, </xsl:if>
            {
                <xsl:if test="position()>1">opposite: true,</xsl:if>
                labels: {
                    style: {
                        color: '#<xsl:value-of select="$color"/>'
                    }
                },
                title: {
                    style: {
                        color: '#<xsl:value-of select="$color"/>'
                    },
            <xsl:choose>
            <xsl:when test="$axis='polar'"/>
            <xsl:when test="$stacking='percent'">
                    text: 'percent',
            </xsl:when>
            <xsl:otherwise>
                    text: '<xsl:value-of select="@unit"/>',
            </xsl:otherwise>
        </xsl:choose>
                    align: 'high'
                }
            }
        </xsl:for-each>
            ],
            series: [
        <xsl:for-each select="t:column_labels/t:column_label[@unit]">
            <xsl:variable name="unit" select="@unit"/>
            <xsl:variable name="unitNumber">
                <xsl:for-each select="$highcharts-units">
                    <xsl:if test="@unit = $unit">
                        <xsl:value-of select="position()"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:if test="position()>1">, </xsl:if>
            {
                name: '<xsl:value-of select="text()"/>',
                type: '<xsl:value-of select="$type"/>',
                yAxis: <xsl:value-of select="$unitNumber - 1"/>,
                data: [
            <xsl:for-each select="ancestor-or-self::t:table/t:row/t:column[count(current()/preceding-sibling::t:column_label)+1]">
                <xsl:if test="position()>1">, </xsl:if>
                <xsl:value-of select="text()"/>
            </xsl:for-each>
                ]
            }
        </xsl:for-each>
            ],

            tooltip: {
                formatter: function() {
                    return ''+ this.series.name +': '+ this.y +'';
                }
            },
            credits: {
                enabled: false
            }
        }
    </xsl:template>

    <xsl:template match="t:section">
        <xsl:param name="comma">,
                </xsl:param>
        <xsl:value-of select="$comma"/><xsl:value-of select="@id"/>: {
        <xsl:apply-templates select="*[1]">
            <xsl:with-param name="comma"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="*[position()>1]"/>
        <xsl:text>}</xsl:text>
    </xsl:template>
    <xsl:template match="t:option[number(text())=text()]">
        <xsl:param name="comma">,
                </xsl:param>
        <xsl:value-of select="$comma"/><xsl:value-of select="@name"/>: <xsl:value-of select="text()"/>
    </xsl:template>
    <xsl:template match="t:option">
        <xsl:param name="comma">,
                </xsl:param>
        <xsl:value-of select="$comma"/><xsl:value-of select="@name"/>: '<xsl:value-of select="text()"/>'
    </xsl:template>
</xsl:stylesheet>
