<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/config.xsd">
    <xsl:include href="/resource/xsl/head.xsl"/>
    <xsl:include href="/resource/xsl/navbar.xsl"/>
    <xsl:output method="html"/>

    <xsl:template match="/">
        <html>
            <xsl:call-template name="head"/>
            <body>
                <xsl:call-template name="navbar"/>
                <div class="container">
                    <div class="alert alert-success">
                        <h2 class="page-title text-center">Lavoisier is being restarted...</h2>
                        <p>New configuration file has been successfully parsed.
                            The table below shows the list of available data views:</p>
                    </div>
                    <table class="table table-bordered table-striped">
                        <thead>
                            <tr>
                                <th>name</th>
                                <th>description</th>
                                <th>arguments</th>
                            </tr>
                        </thead>
                        <tbody>
                            <xsl:apply-templates select="/cfg:config/cfg:view[not(contains('|Hidden|Monitoring|Configuration|Environment|',
                                concat('|',cfg:info/cfg:category/text(),'|')))]"/>
                        </tbody>
                    </table>
                </div>
            </body>
        </html>
    </xsl:template>

    <xsl:template match="cfg:view">
        <!-- if view not already treated -->
        <xsl:variable name="viewName" select="@name"/>
        <tr>
            <td>
                <xsl:choose>
                    <xsl:when test="not(cfg:info/cfg:label/text())">
                        <xsl:value-of select="$viewName"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="cfg:info/cfg:label"/>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
            <td>
                <xsl:value-of select="cfg:info/cfg:description/text()"/>
            </td>
            <td>
                <xsl:for-each select="cfg:argument">
                    <ul style="list-style-type: none; margin: 0; padding: 0">
                        <li>
                            <xsl:choose>
                                <xsl:when test="@eval">
                                    <b><xsl:value-of select="@name"/></b>: ?
                                </xsl:when>
                                <xsl:when test="text() != ''">
                                    <b><xsl:value-of select="@name"/></b>: <xsl:value-of select="text()"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <font color="red">
                                        <b><xsl:value-of select="@name"/></b>
                                    </font>
                                </xsl:otherwise>
                            </xsl:choose>
                            <xsl:if test="@pattern != '[^/]*'">
                                <i> (<xsl:value-of select="@pattern"/>)</i>
                            </xsl:if>
                        </li>
                    </ul>
                </xsl:for-each>
                <xsl:if test="cfg:argument[not(@path-format = 'none')]">
                    <span class="badge">
                        <xsl:for-each select="cfg:argument">
                            <xsl:if test="@path-format = 'name_value'">
                                <xsl:text>/</xsl:text>
                                <xsl:value-of select="@name"/>
                            </xsl:if>
                            <xsl:if test="not(@path-format = 'none')">
                                <xsl:text>/{</xsl:text>
                                <xsl:value-of select="@name"/>
                                <xsl:if test="text() != ''">
                                    <xsl:text>=</xsl:text>
                                    <xsl:value-of select="text()"/>
                                </xsl:if>
                                <xsl:text>}</xsl:text>
                            </xsl:if>
                        </xsl:for-each>
                    </span>
                </xsl:if>
            </td>
        </tr>
    </xsl:template>
</xsl:stylesheet>